94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
118 lines
3.7 KiB
Python
118 lines
3.7 KiB
Python
from __future__ import annotations
|
|
|
|
from typing import Optional
|
|
|
|
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
|
|
|
|
@triton.jit
|
|
def _gather_rows_kernel(
|
|
idx_ptr,
|
|
s0,
|
|
d0,
|
|
n0,
|
|
s1,
|
|
d1,
|
|
n1,
|
|
s2,
|
|
d2,
|
|
n2,
|
|
s3,
|
|
d3,
|
|
n3,
|
|
HAS3: tl.constexpr,
|
|
BLOCK: tl.constexpr,
|
|
):
|
|
# One program == one (output row, column block). All buffers share the
|
|
# same gather index, so a single launch copies every buffer's row and
|
|
# the per-kernel launch bubbles between the old separate gathers vanish.
|
|
row = tl.program_id(0)
|
|
cb = tl.program_id(1)
|
|
src = tl.load(idx_ptr + row).to(tl.int64)
|
|
cols = cb * BLOCK + tl.arange(0, BLOCK)
|
|
|
|
m0 = cols < n0
|
|
tl.store(d0 + row * n0 + cols, tl.load(s0 + src * n0 + cols, mask=m0), mask=m0)
|
|
|
|
m1 = cols < n1
|
|
tl.store(d1 + row * n1 + cols, tl.load(s1 + src * n1 + cols, mask=m1), mask=m1)
|
|
|
|
m2 = cols < n2
|
|
tl.store(d2 + row * n2 + cols, tl.load(s2 + src * n2 + cols, mask=m2), mask=m2)
|
|
|
|
if HAS3:
|
|
m3 = cols < n3
|
|
tl.store(d3 + row * n3 + cols, tl.load(s3 + src * n3 + cols, mask=m3), mask=m3)
|
|
|
|
|
|
def _row_width(buf: torch.Tensor) -> int:
|
|
"""Flattened per-row element count (trailing dims), 1 for a 1-D buffer."""
|
|
return buf[0].numel() if buf.dim() > 1 else 1
|
|
|
|
|
|
def _empty_like_rows(buf: torch.Tensor, m: int) -> torch.Tensor:
|
|
"""Output buffer for `m` gathered rows of `buf` (same trailing dims/dtype/device)."""
|
|
return torch.empty((m, *buf.shape[1:]), dtype=buf.dtype, device=buf.device)
|
|
|
|
|
|
def gather_spec_extras(
|
|
indices: torch.Tensor,
|
|
topk_p_buf: torch.Tensor,
|
|
topk_index_buf: torch.Tensor,
|
|
output_tokens_buf: torch.Tensor,
|
|
hidden_states_buf: Optional[torch.Tensor],
|
|
):
|
|
"""Gather spec extras (topk_p / topk_index / bonus_tokens / optional hidden
|
|
states) by a shared row index in a single fused Triton launch (one kernel
|
|
for all buffers) instead of one advanced-index gather per buffer.
|
|
`hidden_states_buf` is None when the build does not capture hidden states."""
|
|
# Source buffers are allocated once (torch.empty/full) and only ever mutated
|
|
# in place, so they are guaranteed row-contiguous. `indices` flows from
|
|
# several producers (req_pool_indices, filtered/merged future_indices); the
|
|
# kernel addresses it linearly, so normalize layout here (no-op when already
|
|
# contiguous) to avoid a silent wrong-result on a strided index tensor.
|
|
indices = indices.contiguous()
|
|
m = indices.shape[0]
|
|
has_hidden = hidden_states_buf is not None
|
|
|
|
topk_p = _empty_like_rows(topk_p_buf, m)
|
|
topk_index = _empty_like_rows(topk_index_buf, m)
|
|
bonus_tokens = _empty_like_rows(output_tokens_buf, m)
|
|
hidden_states = _empty_like_rows(hidden_states_buf, m) if has_hidden else None
|
|
if m == 0:
|
|
return topk_p, topk_index, bonus_tokens, hidden_states
|
|
|
|
n0 = _row_width(topk_p_buf)
|
|
n1 = _row_width(topk_index_buf)
|
|
n2 = _row_width(output_tokens_buf)
|
|
n3 = _row_width(hidden_states_buf) if has_hidden else 1
|
|
max_n = max(n0, n1, n2, n3)
|
|
|
|
# Dummy operands for the disabled hidden-states slot: the pointers must be
|
|
# valid even though the kernel never dereferences them (gated off by HAS3).
|
|
s3 = hidden_states_buf if has_hidden else indices
|
|
d3 = hidden_states if has_hidden else indices
|
|
|
|
block = min(1024, triton.next_power_of_2(max_n))
|
|
grid = (m, triton.cdiv(max_n, block))
|
|
_gather_rows_kernel[grid](
|
|
indices,
|
|
topk_p_buf,
|
|
topk_p,
|
|
n0,
|
|
topk_index_buf,
|
|
topk_index,
|
|
n1,
|
|
output_tokens_buf,
|
|
bonus_tokens,
|
|
n2,
|
|
s3,
|
|
d3,
|
|
n3,
|
|
HAS3=has_hidden,
|
|
BLOCK=block,
|
|
)
|
|
return topk_p, topk_index, bonus_tokens, hidden_states
|